import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.Iterator;
import javax.imageio.ImageIO;
// Eingefügt von Martin
import java.awt.geom.GeneralPath;

public class Profil {

    GPS_Track gpsTrack;
    Roadbook  roadbook;

    BufferedImage image;

    int imageHeight;
    int imageWidth;

    float altStep = 20;   // 20 m Unterteilung
    int altHeight = 16;   // 16 Pixel pro 20 m
    float kmStep  =  5;   // 5 km Unterteilung
    int kmWidth   = 42;   // 42 Pixel pro 5 km

    float altCorr;
    float deltaAlt = 0;   // Differenz der max/min Höhenmeter
    int altAvg     = 6;   // Höhenglättungs-Faktor

    int xBorder   = 60;   // Linker und rechter Rand
    int yBorder   = 30;


    public Profil (GPS_Track gps_track, Roadbook roadbook) {

        this.gpsTrack = gps_track;
        this.roadbook = roadbook;

        TrackPoint trackPoint;

        deltaAlt = gps_track.maxAlt - gps_track.minAlt;

        // Korrektur für flache Profile, damit Ortsnamen noch passen
        if (deltaAlt < 150) {
            altCorr = 150;
            deltaAlt += altCorr;
        }

        Iterator i = gps_track.iterator ();

        int tpi = 0;

        while (i.hasNext ()) {

            trackPoint = (TrackPoint) i.next ();
            trackPoint.dist = gps_track.getPathLength (gps_track.get (0), trackPoint);

            if ( trackPoint.name.equals ("") ) tpi++;
        }

        // Glättung der Höhenwerte
        float avg;

        for (int k = 1; k <= altAvg; k++) {

            for (int j = 0; j < tpi; j++) {

                trackPoint = gps_track.get (j);

                if (j == 0) {

                    avg = (trackPoint.alt + (gps_track.get (1)).alt) / 2;
                    trackPoint.alt = avg;

                } else if (j == tpi - 1) {

                    avg = ((gps_track.get (j - 1)).alt + trackPoint.alt) / 2;
                    trackPoint.alt = avg;

                } else {

                    avg = ((gps_track.get (j - 1)).alt + trackPoint.alt + (gps_track.get (j + 1)).alt) / 3;
                    trackPoint.alt = avg;
                }
            }
        }
    }

    void drawImage (String filePath) {

        TrackPoint tp;

        imageWidth = (int) Math.round (gpsTrack.length / kmStep) * kmWidth + 2 * xBorder;
        imageHeight = Math.round (deltaAlt / altStep) * altHeight + 2 * yBorder;

        image = new BufferedImage (imageWidth, imageHeight, BufferedImage.TYPE_INT_RGB);

        Graphics2D g2 = (Graphics2D) image.getGraphics ();

        g2.setBackground (Color.WHITE);
        g2.clearRect (0, 0, imageWidth, imageHeight);

        g2.setColor (Color.LIGHT_GRAY);

        // Eingefügt von Martin (Anfang)
        int maxAltImg = 0;
        for (int i = -100; i < 3500; i += altStep) {
          if ((i >= gpsTrack.minAlt) && (i <= gpsTrack.maxAlt + altCorr)) {
            maxAltImg = i;
          }
        }
        int maxAltImgLen = (int) g2.getFontMetrics().getStringBounds(Integer.toString(maxAltImg), g2).getWidth();
        // Eingefügt von Martin (Ende)
        
        for (int i = -100; i < 3500; i += altStep) {

            if ((i >= gpsTrack.minAlt) && (i <= gpsTrack.maxAlt + altCorr)) {

                int y = Math.round (
                        (gpsTrack.maxAlt + altCorr - i) /
                                (gpsTrack.maxAlt + altCorr - gpsTrack.minAlt) *
                                (imageHeight - 2 * yBorder)
                ) + yBorder;

                g2.drawLine (xBorder / 2, y, imageWidth - xBorder / 2, y);

                String s = (new Integer (i)).toString ();
                // Auskommentiert von Martin
                //g2.drawString (s, 3, y + 4);
                
                // Eingefügt von Martin (Anfang)
                int sLen = (int) g2.getFontMetrics().getStringBounds(s, g2).getWidth();
                g2.drawString (s, 3 + maxAltImgLen - sLen, y + 4);
                // Eingefügt von Martin (Ende)
                
                g2.drawString (s, imageWidth - 23, y + 4);
            }
        }

        for (int i = 0; i < gpsTrack.length + kmStep; i += kmStep) {

            String s = Integer.toString (i);
            int x = Math.round ((float) (i / gpsTrack.length) * (imageWidth - 2 * xBorder)) + xBorder;
            // Auskommentiert von Martin
            //g2.drawString (s, x - 2, yBorder);
            
            // Eingefügt von Martin (Anfang)
            int sLen = (int) g2.getFontMetrics().getStringBounds(s, g2).getWidth();
            int start = x - sLen/2;
            g2.drawString (s, start, yBorder);
            // Eingefügt von Martin (Ende)
            
            g2.drawLine (x, yBorder + 4, x, imageHeight - yBorder / 2);
        }


        g2.setStroke (new BasicStroke (6, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND));
        g2.setColor (Color.ORANGE);
        
        // Eingefügt von Martin (Anfang)
        int x1Points[] = new int[gpsTrack.size()];
        int y1Points[] = new int[gpsTrack.size()];
        // Eingefügt von Martin (Ende)

        for (int i = 0; i < gpsTrack.size (); i++) {

            tp = gpsTrack.get (i);

            if (tp.name.equals ("")) {

                float dist = tp.dist;

                int x = Math.round ((float) (dist / gpsTrack.length) * (imageWidth - 2 * xBorder)) + xBorder;
                int y = Math.round ((gpsTrack.maxAlt + altCorr - tp.alt) / (gpsTrack.maxAlt + altCorr - gpsTrack.minAlt) * (imageHeight - 2 * yBorder)) + yBorder;

                // Auskommentiert von Martin
                //g2.drawLine (x, y, x, imageHeight - yBorder / 2);
                
                // Eingefügt von Martin (Anfang)
                x1Points[i] = x;
                y1Points[i] = y;
                // Eingefügt von Martin (Ende)
            }
        }
        
        // Eingefügt von Martin (Anfang)
        GeneralPath polygon = new GeneralPath(GeneralPath.WIND_EVEN_ODD, x1Points.length);
        polygon.moveTo(x1Points[0] - 3, imageHeight - yBorder/2 + 3);
        polygon.lineTo(x1Points[1] - 3, y1Points[1]);
        for (int i = 2; i < x1Points.length - 1; i++) {
          polygon.lineTo(x1Points[i], y1Points[i]);
        };
        polygon.lineTo(x1Points[x1Points.length - 1] + 3, y1Points[x1Points.length - 1]);
        polygon.lineTo(x1Points[x1Points.length - 1] + 3, imageHeight - yBorder/2 + 3);
        polygon.closePath();
        g2.fill(polygon);
        // Eingefügt von Martin (Ende)

        g2.setStroke (new BasicStroke (6, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND));
        g2.setColor (Color.RED);
        
        // Eingefügt von Martin (Anfang)
        for (int i = 0; i < x1Points.length - 1; i++) {
          g2.drawLine(x1Points[i], y1Points[i], x1Points[i + 1], y1Points[i + 1]);
        }
        // Eingefügt von Martin (Ende)

        // Auskommentiert von Martin (Anfang)
        //int xPrev = -1;
        //int yPrev = -1;
        //
        //for (int i = 0; i < gpsTrack.size (); i++) {
        //
        //    tp = gpsTrack.get (i);
        //
        //    if (tp.name.equals ("")) {
        //
        //        float dist = tp.dist;
        //
        //        int x = Math.round ((float) (dist / gpsTrack.length) * (imageWidth - 2 * xBorder)) + xBorder;
        //        int y = Math.round ((gpsTrack.maxAlt + altCorr - tp.alt) / (gpsTrack.maxAlt + altCorr - gpsTrack.minAlt) * (imageHeight - 2 * yBorder)) + yBorder;
        //
        //        if (i == 0) g2.drawLine (x, y, x, y);
        //        else g2.drawLine (xPrev, yPrev, x, y);
        //
        //        xPrev = x;
        //        yPrev = y;
        //    }
        //}
        // Auskommentiert von Martin (Ende)

        g2.setColor (Color.BLACK);

        AffineTransform fontAT = new AffineTransform ();
        fontAT.rotate (Math.toRadians (270));
        Font font = new Font ("ARIAL", Font.PLAIN, 16);
        Font rotatedFont = font.deriveFont (fontAT);
        g2.setFont (rotatedFont);

        for (int i = 0; i < roadbook.size (); i++) {
            PlacePoint pp = roadbook.get (i);

            int j = gpsTrack.getClosestPoint (pp);
            double dist = gpsTrack.getPathLength (gpsTrack.get (0), gpsTrack.get (j));

            int x = Math.round ((float) (dist / gpsTrack.length) * (imageWidth - 2 * xBorder)) + xBorder;
            //g2.drawLine(x, 0, x, height);
            g2.drawString (pp.name, x + 4, imageHeight - yBorder / 2);
        }
        try {
            ImageIO.write (image, "png", new File (filePath));
            System.out.println ("   Profile " + filePath + " generated");
        } catch (IOException ex) {
            ex.printStackTrace ();
        }
    }

    void drawThumb (String filePath) {

        xBorder = 0;
        yBorder = 0;

        imageWidth = 196;
        imageHeight = 120;

        image = new BufferedImage (imageWidth, imageHeight, BufferedImage.TYPE_INT_RGB);

        TrackPoint tp;

        Graphics2D g2 = (Graphics2D) image.getGraphics ();

        g2.setBackground (Color.WHITE);
        g2.clearRect (0, 0, imageWidth, imageHeight);

        g2.setStroke (new BasicStroke (2, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND));
        g2.setColor (Color.ORANGE);

        // Eingefügt von Martin (Anfang)
        int x1Points[] = new int[gpsTrack.size()];
        int y1Points[] = new int[gpsTrack.size()];
        // Eingefügt von Martin (Ende)

        for (int i = 0; i < gpsTrack.size (); i++) {
            tp = gpsTrack.get (i);
            if (tp.name.equals ("") || tp.name.equals ("TP")) {
                float dist = tp.dist;

                int x = Math.round ((float) (dist / gpsTrack.length) * (imageWidth - 2 * xBorder)) + xBorder;
                int y = Math.round ((gpsTrack.maxAlt + altCorr - tp.alt) / (gpsTrack.maxAlt + altCorr - gpsTrack.minAlt) * (imageHeight - 2 * yBorder)) + yBorder;

                // Auskommentiert von Martin
                //g2.drawLine (x, y, x, imageHeight - yBorder / 2);

                // Eingefügt von Martin (Anfang)
                x1Points[i] = x;
                y1Points[i] = y;
                // Eingefügt von Martin (Ende)
            }
        }

        // Eingefügt von Martin (Anfang)
        GeneralPath polygon = new GeneralPath(GeneralPath.WIND_EVEN_ODD, x1Points.length);
        polygon.moveTo(xBorder, imageHeight - yBorder/2);
        for (int i = 0; i < x1Points.length; i++) {
          polygon.lineTo(x1Points[i], y1Points[i]);
        };
        polygon.lineTo(imageWidth - xBorder/2, imageHeight - yBorder/2);
        polygon.closePath();
        g2.fill(polygon);
        // Eingefügt von Martin (Ende)

        g2.setStroke (new BasicStroke (2, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND));
        g2.setColor (Color.RED);

        // Eingefügt von Martin (Anfang)
        for (int i = 0; i < x1Points.length - 1; i++) {
          g2.drawLine(x1Points[i], y1Points[i], x1Points[i + 1], y1Points[i + 1]);
        }
        // Eingefügt von Martin (Ende)

        // Auskommentiert von Martin (Anfang)
        //int xPrev = -1;
        //int yPrev = -1;
        //
        //for (int i = 0; i < gpsTrack.size (); i++) {
        //    tp = gpsTrack.get (i);
        //    if (tp.name.equals ("") || tp.name.equals ("TP")) {
        //        float dist = tp.dist;
        //
        //        int x = Math.round ((float) (dist / gpsTrack.length) * (imageWidth - 2 * xBorder)) + xBorder;
        //        int y = Math.round ((gpsTrack.maxAlt + altCorr - tp.alt) / (gpsTrack.maxAlt + altCorr - gpsTrack.minAlt) * (imageHeight - 2 * yBorder)) + yBorder;
        //
        //        if (i == 0) g2.drawLine (x, y, x, y);
        //        else g2.drawLine (xPrev, yPrev, x, y);
        //
        //        xPrev = x;
        //        yPrev = y;
        //    }
        //}
        // Auskommentiert von Martin (Ende)

        g2.setColor (Color.BLACK);
        Font font = new Font ("ARIAL", Font.PLAIN, 11);
        g2.setFont (font);

        String s = (Math.round (new Float (gpsTrack.maxAlt))) + " m";
        g2.drawString (s, 2, 10);

        s = (Math.round (new Float (gpsTrack.minAlt))) + " m";
        g2.drawString (s, 2, imageHeight - 2);

        try {

            ImageIO.write (image, "png", new File (filePath));

            System.out.println ("   Profile " + filePath + " generated");

        } catch (IOException ex) {
            ex.printStackTrace ();
        }
    }
}